### -*- coding: utf-8 -*- #############################################
# Разработано компанией Стерх (http://sterch.net/)
# Все права защищены, 2008
#
# Developed by Sterch (http://sterch.net/)
# All right reserved, 2008
#######################################################################

"""Text processing functions

$Id: text.py 14403 2010-04-29 14:34:40Z maxp $
"""
__author__  = "Polshcha Maxim (maxp@sterch.net)"
__license__ = "<undefined>" # необходимо согласование
__version__ = "$Revision: 14403 $"
__date__ = "$Date: 2010-04-29 17:34:40 +0300 (Чт, 29 апр 2010) $"

import csv
import os.path
import re

__MY__PATH__ = os.path.dirname(os.path.abspath(__file__))
glEntities = dict([p for p in csv.reader(open(os.path.join(__MY__PATH__,"entities.csv"),"rU")) ]) 

def striptags(text):
    """ strips tags from the text"""
    return re.sub("<[^>]+>", " ", text)

def replace_html_entities(text):
    """ replaces HTML entities from the text """
    global glEntities
    # replace numeric entities
    _numentities = set(re.findall("(&#\d+;)", text))
    for entity in _numentities:
        code = entity[2:-1]
        text = text.replace(entity, unichr(int(code)))
    # replace character entities
    _entities = set(re.findall("(&[a-zA-Z0-9]+;)", text))
    for entity in _entities:
        literal = entity[1:-1] 
        if literal in glEntities:
            try:
                text = text.replace(entity, unichr(int("0x%s" % glEntities[literal], 16)))
            except Exception:
                pass 
    return text

def normalize(s):
    ss = s
    _javascript = re.findall("(<script.*?>.*?</script>)", ss, re.MULTILINE|re.DOTALL)
    for script in _javascript:
        ss = ss.replace(script,' ')
    _styles = re.findall("(<style.*?>.*?</style>)", ss, re.MULTILINE|re.DOTALL)
    for style in _styles:
        ss = ss.replace(style,' ')
    ss =  replace_html_entities(striptags(ss))
    ss = ss.replace("\r",' ')
    ss = ss.replace("\n",' ')
    ss = ss.replace("\t",' ')
    ss = ss.strip()
    while '  ' in ss: ss=ss.replace('  ', ' ')
    return ss

def tofilename(name):
    n = name
    illegal=":,|<>\\/\"'~`!@#$%^&*()\n\r\t?;"
    for s in illegal : n = n.replace(s,'')
    return n


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Стерх (http://sterch.net/)
# Все права защищены, 2008
#
# Developed by Sterch (http://sterch.net/)
# All right reserved, 2008
#######################################################################

""" Headers dictionary

$Id: headers.py 14192 2010-04-17 08:44:49Z maxp $
"""
__author__  = "Polscha Maxim (maxp@sterch.net)"
__license__ = "<undefined>" # необходимо согласование
__version__ = "$Revision: 14192 $"
__date__ = "$Date: 2010-04-17 11:44:49 +0300 (Сб, 17 апр 2010) $"

from random import choice, randint
from copy import copy
import string

browser_headers = [
 [
                   ('User-Agent', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.21 Safari/532.0'),
                   ('Accept', 'application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5'),
                   ('Accept-Language', 'en-US;q=0.8,en;q=0.6'),
                   ('Accept-Charset', 'utf-8;q=0.8,latin-1;q=0,6,*;q=0.3'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) (KHTML, like Gecko)'),
                   ('Accept', 'application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5'),
                   ('Accept-Language', 'en-US'),
                   ('Accept-Charset', 'utf-8;q=0.9,latin-1;q=0,7,*;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; uk; rv:1.9.1.3) Gecko/20090824 Firefox/3.5.3 GTB5 (.NET CLR 3.5.30729)'),
                   ('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8,text/vnd.wap.wml;q=0.6'),
                   ('Accept-Language', 'ru,uk;q=0.8,en;q=0.7,de;q=0.5,ru-ru;q=0.3,en-us;q=0.2'),
                   ('Accept-Charset', 'utf-8;q=0.8,*;q=0.7'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU) AppleWebKit/530.19.2 (KHTML, like Gecko) Version/4.0.2 Safari/530.19.1'),
                   ('Accept', 'application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5'),
                   ('Accept-Language', 'en-US'),
                   ('Accept-Charset', 'iso-8859-1, utf-8'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.80 (Windows NT 5.1; U; en)'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en;q=0.9,ru;q=0.1'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, utf-16, *;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.80 (Windows NT 5.1; U; en) Presto/2.2.15 Version/10.00'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en-US,en;q=0.9,en;q=0.8'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, utf-16, *;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'ru,en-US;q=0.5'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, utf-16, *;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en-US;q=0.9'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, utf-16, *;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en-US;q=0.7'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, utf-16, *;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) (KHTML, like Gecko)'),
                   ('Accept', 'application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5'),
                   ('Accept-Language', 'en-US'),
                   ('Accept-Charset', 'utf-8;q=0.8,iso-8859-1;q=0.6,*;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/5.0 (Windows; Windows NT 5.1; US; rv:1.9.1) Gecko/20090720 Firefox/3.5.1'),
                   ('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.8,*/*;q=0.4,text/vnd.wap.wml;q=0.2'),
                   ('Accept-Language', 'en;q=0.7,en-us;q=0.2'),
                   ('Accept-Charset', 'utf-8, iso-8859-1;q=0.8,*;q=0.6'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/5.0 (Windows NT 5.1; en-US) AppleWebKit/530.19.2 Version/4.0.1 Safari/530.17.0'),
                   ('Accept', 'application/xml,application/xhtml+xml,text/html;q=0.8,text/plain;q=0.7,image/png,*/*;q=0.1'),
                   ('Accept-Language', 'en-US, en;q=0.9'),
                   ('Accept-Charset', 'utf-8, iso-8859-1;q=0.9'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.85 (Windows NT 5.1; en-US)'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en;q=0.9'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.5, utf-16, *;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.88 (Windows NT 5.1; en) Presto/2.1.13 Version/9.00'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en-US;q=0.9,en;q=0.3'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50700;'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en-US;q=0.8'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.4'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/3.9)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en-US;q=0.9'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, *;q=0.3'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1;  .NET CLR 2.0.507123;)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en,en-US;q=0.7'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
  [                 
                   ('User-Agent', 'Opera/9.65 (Windows; Windows NT 5.1; en) Version/9.00'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en-US;q=0.9,en;q=0.3'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows; Windows NT 5.1; .NET'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en-US;q=0.9'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.4'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/3.7; .NET)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en-US;q=0.9'),
                   ('Accept-Charset', 'iso-8859-1, utf-8, *;q=0.3'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows; Windows NT 5.1; US)'),
                   ('Accept', 'image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, application/x-silverlight, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'),
                   ('Accept-Language', 'en,en-US;q=0.7'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.68 (Windows; Windows; Windows NT 5.1; en)'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en-US;q=0.9,en;q=0.3'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.68 ( Windows NT 5.1; en) Version/9.00'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en-US;q=0.9,en;q=0.3'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
 [                 
                   ('User-Agent', 'Opera/9.67 (Windows; Windows NT 5.1; en) Version/9.00'),
                   ('Accept', 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1'),
                   ('Accept-Language', 'en-US;q=0.9,en;q=0.3'),
                   ('Accept-Charset', 'iso-8859-1, utf-8;q=0.1'),
                   ('Keep-Alive', '300'),
                   ('Connection', 'keep-alive')
 ],
]

def getheaders():
    """ Returns random browser headers """
    headers = copy(choice(browser_headers))
    ua, value = headers[0]
    name = "".join([choice(string.letters) for l in xrange(0,randint(3,10))])
    major = str(randint(0,9))
    middle =  str(randint(0,20))
    minor =  str(randint(0,150))
    tail = "; %s v%s.%s.%s;" % (name, major, middle, minor)
    index = value.find(")")
    if index > 0:
        value = value[:index] + tail + value[index:]  
    else:
        value = value + tail
    headers[0] = (ua, value)
    headers.append(("X-Forwarded-For", ".".join(map(str,[10,randint(0,253),randint(0,253),randint(1,253)]))))
    
    return headers 


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Стерх (http://sterch.net/)
# Все права защищены, 2008
#
# Developed by Sterch (http://sterch.net/)
# All right reserved, 2008
#######################################################################

""" Captcha resolving tools with the help of http://decapthcer.com/

$Id: decaptcher.py 14192 2010-04-17 08:44:49Z maxp $
"""
__author__  = "Polscha Maxim (maxp@sterch.net)"
__license__ = "<undefined>" # необходимо согласование
__version__ = "$Revision: 14192 $"
__date__ = "$Date: 2010-04-17 11:44:49 +0300 (Сб, 17 апр 2010) $"

from opener import Client

class DecaptcherException(Exception):
    """ Captcha was not solved """

def decaptcher_solve(username, password, captcha, filename=None, client=None):
    """ capthca --- value of capthca to solve """
    if not client:
        c = Client(noproxy=True)
    else: 
        c = client
    url = "http://poster.decaptcher.com/"
    if not filename:
        fname = "captcha"
    else:
        fname = "filename"
    fields = {'function':'picture2', 'username':username, 'password':password, 'pict_to':'0', 'pict_type':'0', 'submit':'Send'}
    resolved = c.post_multipart(url, fields.items(), [('pict',fname, captcha)])
    result_code = resolved.split("|")[0]
    if result_code == "0":
        return resolved.split("|")[-1]
    raise DecaptcherException("Decaptcher return code: %s" % result_code)


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Стерх (http://sterch.net/)
# Все права защищены, 2008
#
# Developed by Sterch (http://sterch.net/)
# All right reserved, 2008
#######################################################################
# $Id: decorators.py 14755 2010-05-26 12:12:56Z maxp $
#######################################################################

""" useful decorators

$Id: decorators.py 14755 2010-05-26 12:12:56Z maxp $
"""
__author__  = "Polscha Maxim (maxp@sterch.net)"
__license__ = "<undefined>" # необходимо согласование
__version__ = "$Revision: 14755 $"
__date__ = "$Date: 2010-05-26 15:12:56 +0300 (Ср, 26 май 2010) $"

def synchronized(lock):
    """ Synchronization decorator. """

    def wrap(f):
        def new_function(*args, **kw):
            lock.acquire()
            try:
                return f(*args, **kw)
            finally:
                lock.release()
        return new_function
    return wrap


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Стерх (http://sterch.net/)
# Все права защищены, 2008
#
# Developed by Sterch (http://sterch.net/)
# All right reserved, 2008
#######################################################################
# $Id: __init__.py 14592 2010-05-17 11:00:42Z maxp $
#######################################################################

# Make it a Python package
from opener import createOpener, readpage, proxies, Client, BaseCaptchaAwareClient
from output import start_chunked_stdout, stop_chunked_stdout
from queue import pickleQueue, unpickleQueue, processQueue, SyncList, DuplicateValueError
from queue import DumpingGetQueue, DumpingPutQueue
from text import replace_html_entities, striptags, normalize, tofilename
from writer import CSVWriter, SimpleCSVWriter
from worker import Worker, workers_group_factory, filter_dead_workers


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Стерх (http://sterch.net/)
# Все права защищены, 2008
#
# Developed by Sterch (http://sterch.net/)
# All right reserved, 2008
#######################################################################

""" Page downloading tools

$Id: opener.py 14239 2010-04-20 10:05:00Z maxp $
"""
__author__  = "Polscha Maxim (maxp@sterch.net)"
__license__ = "<undefined>" # необходимо согласование
__version__ = "$Revision: 14239 $"
__date__ = "$Date: 2010-04-20 13:05:00 +0300 (Вт, 20 апр 2010) $"

from config import MAXREADTRIES, DELAY
from cookielib import CookieJar
from handlers import BindableHTTPHandlerFactory
from headers import getheaders
from random import choice
from time import sleep

import mimetypes
import os.path
import urllib
import urllib2 

cjar = CookieJar()

# Load proxies
__MY__PATH__ = os.path.dirname(os.path.abspath(__file__))

try:
    f = open(os.path.join(__MY__PATH__, "proxies.txt"),"rt")
    proxies = map(lambda s:s.strip(), f.readlines())
    f.close()
    proxies = filter(lambda s:not s.startswith("#"), proxies)
except Exception, ex:
    print ex
    proxies = []

# Load available IPS
try:
    f = open(os.path.join(__MY__PATH__, "ips.txt"),"rt")
    ips = map(lambda s:s.strip(), f.readlines())
    f.close()
    ips = filter(lambda s:not s.startswith("#"), ips)
except Exception, ex:
    print ex
    ips = []

    
def createOpener(cookies=None, headers=None, _proxies=None):
    global ips
    handlers = []
    if _proxies:
        proxy_support = urllib2.ProxyHandler(_proxies)
        handlers.append(proxy_support)
    
    if ips:
        handlers.append(BindableHTTPHandlerFactory(choice(ips)))
        
    if cookies is not None :
        c = urllib2.HTTPCookieProcessor()
        c.cookiejar = cookies
        handlers.append(c)
    
    opener = urllib2.build_opener(*handlers)
    
     
    if headers:
        opener.addheaders = headers
    else:
        opener.addheaders = getheaders()
    return opener

def readpage(url, data=None, cookies=None, headers=None, _proxies=None, needURL=False):
    """ url --- url to read
        data --- data to be POSTed. if dictionary --- in will be encoded.
        needURL --- if set to True readpage returns tuple (data, url) where url is real reading url after redirects.
    """
    global cjar 
    ntries = 0
    downloaded = False
    c = cookies 
    if cookies is not None:
        c = cookies
    else:
        c = cjar
    opener = createOpener(cookies=c, headers=headers, _proxies = _proxies)
    realURL=''
    while not downloaded and ntries < MAXREADTRIES:
        try: 
            if type(data) is dict:
                topost = urllib.urlencode(data)
            else:
                topost = data
            request = urllib2.Request(url, topost)   
            f = opener.open(request)
            if needURL: realURL = f.geturl()
            page = f.read()
            f.close()
            downloaded = True
            opener.close()
        except Exception, ex:
            if type(ex) == urllib2.HTTPError:
                print "ERROR: Can't read %s. Error %d" % (url, ex.code)
                msg = "Error %d" % ex.code
                if needURL:
                    return (msg, url)
                else:
                    return msg
            else:
                print "ERROR: network error (%s)" % url, ex
            opener.close()
            sleep(DELAY)
            ntries += 1
            opener = createOpener(cookies=c, headers=headers, _proxies = _proxies)
            
    if not downloaded : 
        print "ERROR: Can't download page %s after %d tries. %s" % (url, ntries,ex)
        page = ""
     
    if needURL:
        return (page, realURL)
    else:
        return page

def encode_multipart_formdata(fields, files):
    """
    fields is a sequence of (name, value) elements for regular form fields.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files
    Return (content_type, body) ready for httplib.HTTP instance
    """
    BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
    CRLF = '\r\n'
    L = []
    for (key, value) in fields:
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"' % key)
        L.append('')
        L.append(value)
    for (key, filename, value) in files:
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
        L.append('Content-Type: %s' % get_content_type(filename))
        L.append('')
        L.append(value)
    L.append('--' + BOUNDARY + '--')
    L.append('')
    body = CRLF.join(L)
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
    return content_type, body

def get_content_type(filename):
    """ Determines content type of the file """
    return mimetypes.guess_type(filename)[0] or 'application/octet-stream'

class Client(object):
    """ Simple browser emulator implements following policy:
        every read uses same cookiejar, same proxy and same browser headers.
    """
    def __init__(self, cookies=None, headers=None, _proxies=None, noproxy=False):
        global proxies
         
        if cookies is not None:
            self.cookies = cookies
        else:
            self.cookies = CookieJar()
        if headers:
            self.headers = headers 
        else:
            self.headers = getheaders()
        if not noproxy: 
            if _proxies:
                self.proxies = _proxies
            elif proxies:
                self.proxies = {'http' : choice(proxies), 'https' : choice(proxies) }
            else:
                self.proxies = None
        else:
            self.proxies = None
        self.lastURL = None
    
    def readpage(self, url, data=None):
        data, realurl = readpage(url, data=data, 
                        cookies = self.cookies,
                        headers = self.headers,
                        _proxies = self.proxies,
                        needURL = True)
        self.lastURL = realurl
        return data
    
    def getrealurl(self, url):
        """ returns real url after redirects """
        opener = createOpener(cookies = self.cookies,
                    headers = self.headers,
                    _proxies = self.proxies)
        
        f = opener.open(url)
        realURL = f.geturl()
        f.close()
        return realURL

    def post_multipart(self, url, fields, files):
        """
        Post fields and files to an http host as multipart/form-data.
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, value) elements for data to be uploaded as files
        Return the server's response page.
        """
        content_type, body = encode_multipart_formdata(fields, files)
        global cjar 
        ntries = 0
        downloaded = False
        c = self.cookies 
        if self.cookies is not None:
            c = self.cookies
        else:
            c = cjar
        opener = createOpener(cookies=c, headers=self.headers, _proxies = self.proxies)
        headers = {'Content-Type': content_type,
                   'Content-Length': str(len(body))}
        request = urllib2.Request(url, body, headers)
        
        while not downloaded and ntries < MAXREADTRIES:     
            try:     
                f = opener.open(request)
                page = f.read()
                f.close()
                downloaded = True
                opener.close()
            except Exception, ex:
                if type(ex) == urllib2.HTTPError:
                    print "ERROR: Can't read %s. Error %d" % (url, ex.code)
                else:
                    print "ERROR: network error (%s)" % url, ex
                opener.close()
                sleep(DELAY)
                ntries += 1
                opener = createOpener(cookies=c, headers=headers, _proxies = self.proxies)
        if not downloaded : 
            print "ERROR: Can't download page %s after %d tries. %s" % (url, ntries,ex)
            page = ""
        return page
    
class BaseCaptchaAwareClient(Client):
    """ Base class for clients that aware to solve captcha """
    
    def captcha_required(self, page):
        """ Returns True if cpatche entry required, False otherwise.
            Accepts page content as input.
        """
        raise NotImplemented()
    
    def solve_captcha(self, page):
        """ Returns page content after captcha solving.
            Accepts page content as input.
        """
        raise NotImplemented()
    
    def readpage(self, url, data=None):
        page = super(BaseCaptchaAwareClient, self).readpage(url, data)
        if self.captcha_required(page):
            is_solved = False
            ntries = 0
            while not is_solved and ntries < MAXREADTRIES: 
                try:
                    page = self.solve_captcha(page)
                    is_solved = True
                except Exception, ex:
                    print "ERROR: captcha solving error:",  ex
                sleep(DELAY)
                ntries += 1
            if not is_solved: return ''
        return page